/* Diana.java
 *
 * created: Fri Oct  9 1998
 *
 * This file is part of Artemis
 *
 * Copyright (C) 1998,1999,2000  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/Diana.java,v 1.94 2000/03/22 17:22:35 kmr Exp $
 *
 *  A physicist, an engineer, and a computer scientist were
 *  discussing the nature of God.  Surely a Physicist, said the
 *  physicist, because early in the Creation, God made Light; and you
 *  know, Maxwell's equations, the dual nature of electro-magnetic
 *  waves, the relativist consequences... An Engineer!, said the
 *  engineer, because before making Light, God split the Chaos into
 *  Land and Water; it takes a hell of an engineer to handle that big
 *  amount of mud, and orderly separation of solids from
 *  liquids... The computer scientist shouted: And the Chaos,
 *  where do you think it was coming from, hmm?
 *
 *  ---Anonymous
 *
 */

import diana.components.*;
import diana.*;

import uk.ac.sanger.pathogens.StringVector;
import uk.ac.sanger.pathogens.embl.EntryInformation;
import uk.ac.sanger.pathogens.embl.SimpleEntryInformation;
import uk.ac.sanger.pathogens.embl.Key;
import uk.ac.sanger.pathogens.embl.KeyVector;
import uk.ac.sanger.pathogens.embl.QualifierInfo;
import uk.ac.sanger.pathogens.embl.QualifierInfoVector;
import uk.ac.sanger.pathogens.embl.QualifierInfoException;

import java.io.*;
import java.util.*;

/**
 *  This class contains the method main () for running Diana as a standalone
 *  program.
 *
 *  @author Kim Rutherford
 *  @version $Id: Diana.java,v 1.94 2000/03/22 17:22:35 kmr Exp $
 * */

public class Diana {
  /**
   *  Version String use for banner messages and title bars.
   **/
  public static final String version = "Release 3";

  /**
   *  Main entry point for the stand-alone Diana.
   **/
  public static void main (final String [] args) {
    try {
      db_entry_infomation = makeEntryInformation ();
      extended_entry_infomation =
        makeExtendedEntryInformation (db_entry_infomation);
    } catch (QualifierInfoException e) {
      System.err.println ("could not initialise the embl package: " +
                          e.getMessage ());
      System.exit (1);
    } catch (IOException e) {
      System.err.println ("could not initialise the embl package: " +
                          e.getMessage ());
      System.exit (1);
    }

    final ArtemisMain main_window = new ArtemisMain ();

    main_window.setVisible (true);

    // read the entries given on the command line and in the diana.ini file
    main_window.readArgsAndOptions (args);
  }

  /**
   *  The EntryInformation object to use for EMBL and GENBANK entries.
   **/
  private static EntryInformation extended_entry_infomation;

  /**
   *  The EntryInformation object to use for EMBL and GENBANK entries.
   **/
  private static EntryInformation db_entry_infomation;

  /**
   *  Return the EntryInformation object to use for EMBL and GENBANK entries.
   **/
  public static EntryInformation getDBEntryInformation () {
    return db_entry_infomation;
  }

  /**
   *  The EntryInformation object that has all the information that
   *  getDBEntryInformation () would return, but which has the non-standard
   *  qualifiers and keys added.
   **/
  public static EntryInformation getExtendedEntryInformation () {
    return extended_entry_infomation;
  }

  /**
   *  Return an EntryInformation object that is suitable for EMBL and GENBANK
   *  entries.  The returned EntryInformation object will have the
   *  use_embl_location_format flag set to false.
   **/
  private static EntryInformation makeEntryInformation ()
      throws IOException, QualifierInfoException {
    final InputStream feature_keys_stream =
      getFileFromJar ("etc/feature_keys");

    final InputStream qualifier_types_stream =
      getFileFromJar ("etc/qualifier_types");

    QualifierInfoVector qualifier_info_vector =
      readQualifierInfo (qualifier_types_stream, feature_keys_stream);

    final EntryInformation entry_information = new SimpleEntryInformation ();

    for (int i = 0 ; i < qualifier_info_vector.size () ; ++i) {
      final QualifierInfo qualifier_info =
        qualifier_info_vector.elementAt (i);

      entry_information.addQualifierInfo (qualifier_info);
    }

    entry_information.setEMBLLocationFormat (true);

    return entry_information;
  }

  /**
   *  Return an EntryInformation object that is suitable for EMBL and GENBANK
   *  entries, and has some useful non-standard additions (such as the extra
   *  keys and qualifiers given in the options file).  The returned
   *  EntryInformation object will have the use_embl_location_format flag set
   *  to false.
   **/
  private static EntryInformation
    makeExtendedEntryInformation (final EntryInformation standard_entry_info)
      throws QualifierInfoException {

    final StringVector extra_keys =
      getOptions ().getOptionValues ("extra_keys");

    final QualifierInfoVector extra_qualifiers =
      getOptions ().getExtraQualifiers ();

    final EntryInformation return_entry_information =
      new SimpleEntryInformation (standard_entry_info);

    for (int i = 0 ; i < extra_keys.size () ; ++i) {
      final Key new_key = new Key (extra_keys.elementAt (i));

      return_entry_information.addKey (new_key);
    }

    for (int i = 0 ; i < extra_qualifiers.size () ; ++i) {
      final QualifierInfo new_qualifier_info = extra_qualifiers.elementAt (i);

      return_entry_information.addQualifierInfo (new_qualifier_info);
    }

    return_entry_information.setEMBLLocationFormat (false);

    return return_entry_information;
  }

  /**
   *  Read the possible feature key and qualifier names and types from the two
   *  given streams (see etc/feature_keys and etc/qualifier_types for details
   *  on the formats).
   **/
  private static QualifierInfoVector
    readQualifierInfo (final InputStream qualifier_types_stream,
                       final InputStream feature_keys_stream)
      throws IOException {

    final QualifierInfoVector return_vector = new QualifierInfoVector ();

    Properties feature_properties = new Properties ();
    final Properties qualifier_properties = new Properties ();

    feature_properties.load (feature_keys_stream);
    qualifier_properties.load (qualifier_types_stream);

    // parse the feature_properties

    {
      final Properties new_feature_properties = new Properties ();

      final Enumeration feature_enum = feature_properties.propertyNames ();

      while (feature_enum.hasMoreElements ()) {
        String current_feature_name = (String) feature_enum.nextElement ();

        final StringVector property_values =
          Options.getPropertyValues (feature_properties, current_feature_name);

        new_feature_properties.put (current_feature_name, property_values);
      }

      feature_properties = new_feature_properties;
    }

    final Enumeration qualifier_enum = qualifier_properties.propertyNames ();

    while (qualifier_enum.hasMoreElements ()) {

      String current_qualifier_name = (String) qualifier_enum.nextElement ();

      final StringVector current_qualifier_values =
        Options.getPropertyValues (qualifier_properties,
                                   current_qualifier_name);


      final boolean once_only =
        current_qualifier_values.elementAt (0).equals ("yes");

      final String type_string = current_qualifier_values.elementAt (1);

      // find the keys for which this qualifier name is valid or required

      final KeyVector valid_keys = new KeyVector ();
      final KeyVector required_keys = new KeyVector ();

      final Enumeration features_enum = feature_properties.propertyNames ();

      while (features_enum.hasMoreElements ()) {
        final String current_key_string = (String)features_enum.nextElement ();

        final Key current_key = new Key (current_key_string);

        final StringVector current_feature_qualifiers =
          (StringVector) feature_properties.get (current_key_string);

        if (current_feature_qualifiers.contains (current_qualifier_name)) {
          valid_keys.add (current_key);
        } else {
          if (current_feature_qualifiers.contains ("@" +
                                                   current_qualifier_name)) {
            valid_keys.add (current_key);
            required_keys.add (current_key);
          }
        }
      }

      final int type = QualifierInfo.getQualifierTypeID (type_string);

      final QualifierInfo qualifier_info =
        new QualifierInfo (current_qualifier_name, type, valid_keys,
                           required_keys, once_only);

      return_vector.add (qualifier_info);
    }

    return return_vector;
  }

  /**
   *  Return the reference of the global options object for this program.
   **/
  static public Options getOptions () {
    return Options.getOptions ();
  }

  /**
   *  Return true if and only if we are running in an applet.
   **/
  public static boolean isApplet () {
    return false;
  }

  /**
   *  Return true if and only if we are not running in an applet.
   **/
  public static boolean isStandAlone () {
    return true;
  }

  /**
   *  This method is a hack to get around a bug in Netscape - it is
   *  implemented differently by DianaApplet.
   **/
  public static InputStream getFileFromJar (final String file_name) {
    return Diana.class.getResourceAsStream (file_name);
  }
}
