/* MUMmerComparisonData.java
 *
 * created: Thu Jul 15 1999
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1999  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/MUMmerComparisonData.java,v 1.1 1999/07/21 13:43:24 kmr Exp $
 */

package diana;

import uk.ac.sanger.pathogens.embl.LinePushBackReader;

import java.io.*;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 *  This class implements the ComparisonData interface for MUMmer output.
 *
 *  @author Kim Rutherford
 *  @version $Id: MUMmerComparisonData.java,v 1.1 1999/07/21 13:43:24 kmr Exp $
 **/

public class MUMmerComparisonData implements ComparisonData {
  /**
   *  Create a new MUMmerComparisonData object by reading from the given
   *  LinePushBackReader.
   **/
  public MUMmerComparisonData (final LinePushBackReader stream)
      throws IOException {
    final String first_line = stream.readLine ();

    final Vector align_match_vector = new Vector ();

    while (true) {
      final String line = stream.readLine ();

      if (line == null) {
        break;
      }

      if (line.startsWith (">")) {
        break;
      }

      final StringTokenizer tokenizer = new StringTokenizer (line, " ");

      if (tokenizer.countTokens () < 6) {
        final String message =
          "unable to understand this line: " + line;
        throw new ComparisonDataParseException (message);
      }
      
      final String first_token = tokenizer.nextToken ();
      final String second_token = tokenizer.nextToken ();
      final String third_token = tokenizer.nextToken ();


      final int first_number = Integer.valueOf (first_token).intValue ();
      final int second_number = Integer.valueOf (second_token).intValue ();
      final int third_number = Integer.valueOf (third_token).intValue ();

      final AlignMatch new_match =
        new AlignMatch (second_number, second_number + third_number,
                        first_number, first_number + third_number);
      align_match_vector.addElement (new_match);
    }

    matches = new AlignMatch [align_match_vector.size ()];

    for (int i = 0 ; i < matches.length ; ++i) {
      matches[i] = (AlignMatch) align_match_vector.elementAt (i);
    }
  }

  /**
   *  Return an array containing all the AlignMatch objects for this
   *  comparison.
   **/
  public AlignMatch [] getAlignments () {
    return matches;
  }

  /**
   *  This is the array of matches created by the constructor.
   **/
  private final AlignMatch [] matches;
}

