/* QualifierVector.java
 *
 * created: Fri Mar 12 1999
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1998,1999,2000  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/QualifierVector.java,v 1.4 2000/01/24 17:04:17 kmr Exp $
 **/

package diana;

import uk.ac.sanger.pathogens.embl.Qualifier;

import java.util.Vector;

/**
 *  This class implements a vector of embl.Qualifier objects.
 *
 *  @author Kim Rutherford
 *  @version $Id: QualifierVector.java,v 1.4 2000/01/24 17:04:17 kmr Exp $
 **/

public class QualifierVector {
  /**
   *  Create a new (empty) vector of embl.Qualifier objects.
   **/
  public QualifierVector () {

  }

  /**
   *  Appends the given Qualifier object to the vector if and only if it isn't
   *  already in the vector.
   **/
  protected void addElement (Qualifier qualifier) {
    if (indexOf (qualifier) == -1) {
      vector.addElement (qualifier);
    }
  }
  
  /**
   *  Appends the given Qualifier object to the vector if and only if it isn't
   *  already in the vector.  (same as addElement ()).
   **/
  public void add (Qualifier qualifier) {
    addElement (qualifier);
  }
  
  /**
   *  Performs the same function as Vector.elementAt ()
   **/
  public Qualifier elementAt (int index) {
    return (Qualifier) vector.elementAt (index);
  }

  /**
   *  Performs the same function as Vector.removeElement ()
   **/
  protected boolean removeElement (Qualifier qualifier) {
    return vector.removeElement (qualifier);
  }

  /**
   *  Return true if this object contains the given Qualifier.
   **/
  public boolean contains (Qualifier qualifier) {
    if (indexOf (qualifier) == -1) {
      return false;
    } else {
      return true;
    }
  }

  /**
   *  Performs the same function as Vector.removeAllElements ()
   **/
  public void removeAllElements () {
    vector.removeAllElements ();
  }

  /**
   *  Performs the same function as Vector.removeElement ()
   **/
  public int indexOf (Qualifier qualifier) {
    return vector.indexOf (qualifier);
  }

  /**
   *  Performs the same function as Vector.size ()
   **/
  public int size () {
    return vector.size ();
  }
  
  /**
   *  Storage for Qualifier objects.
   **/
  final private Vector vector = new Vector ();
}
