/* EntryFileDialog.java
 *
 * created: Mon Dec  7 1998
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1998,1999,2000  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/components/EntryFileDialog.java,v 1.19 2000/03/02 11:00:25 kmr Exp $
 */

package diana.components;

import uk.ac.sanger.pathogens.*;
import uk.ac.sanger.pathogens.embl.DocumentEntryFactory;
import uk.ac.sanger.pathogens.embl.ReadFormatException;
import uk.ac.sanger.pathogens.embl.EntryInformation;
import uk.ac.sanger.pathogens.embl.EntryInformationException;
import uk.ac.sanger.pathogens.embl.InvalidRelationException;

import java.awt.*;
import java.io.*;

/**
 *  This class is a FileDialog that can read EMBL Entry objects.
 *
 *  @author Kim Rutherford
 *  @version $Id: EntryFileDialog.java,v 1.19 2000/03/02 11:00:25 kmr Exp $
 **/

public class EntryFileDialog extends FileDialog {
  /**
   *  Create a new EntryFileDialog component.
   *  @param owner The component where this dialog was created.
   *  @param entry_group The EntryGroup object where the newly read Entry
   *    should be put.
   **/
  public EntryFileDialog (Frame owner) {
    super (owner, "Select an EMBL file...", FileDialog.LOAD);
    this.owner = owner;
  }

  /**
   *  Return an embl.Entry object representing the file the user has selected
   *  with the dialog or null if the read failed for any reason.
   *  @param entry_information The EntryInformation to use when reading.  This
   *    supplies the list of valid keys and qualifiers
   *  @exception EntryInformationException Thrown if an Entry using the given
   *    EntryInformation object cannot contain the Key, Qualifier or
   *    Key/Qualifier combination of one of the features in the Document.
   **/
  public uk.ac.sanger.pathogens.embl.Entry getEntry (final EntryInformation
                                                       entry_information)
      throws EntryInformationException {
    show ();

    if (getDirectory () == null || getFile () == null) {
      return null;
    }
    
    final File file = new File (getDirectory (), getFile ());
    
    try {

      System.out.println ("trying to open this file: " +  file);

      final FileDocument file_document = new FileDocument (file);

      if (file != null) {

        final uk.ac.sanger.pathogens.embl.Entry new_entry =
          DocumentEntryFactory.makeDocumentEntry (entry_information,
                                                  file_document);

        return new_entry;
      } else {
        new MessageDialog (owner, "creating new entry: " + getFile ());
        System.out.println ("creating new entry: " + getFile ());

        final uk.ac.sanger.pathogens.embl.Entry new_entry =
          DocumentEntryFactory.makeDocumentEntry (file_document);

        return new_entry;
      }
    } catch (ReadFormatException e) {
      final String message =
        "failed to open " + file + ": " + e.getMessage () +
        " at line: " + e.getLineNumber ();
      System.out.println (message);
      new MessageDialog (owner, message);
    } catch (IOException e) {
      final String message = "failed to open " + file + ": " + e.getMessage ();
      System.out.println (message);
      new MessageDialog (owner, message);
    }

    return null;
  }

  /**
   *  The Frame reference that was passed to the constructor.
   **/
  private Frame owner = null;
}


