/* StringVector.java
 *
 * created: Fri Jan  1 1999
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1998,1999,2000  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/uk/ac/sanger/pathogens/StringVector.java,v 1.4 2000/02/24 12:52:11 kmr Exp $
 */

package uk.ac.sanger.pathogens;

import java.util.Vector;
import java.util.StringTokenizer;

/**
 *  This class implements a Vector of String objects.
 *
 *  @author Kim Rutherford
 *  @version $Id: StringVector.java,v 1.4 2000/02/24 12:52:11 kmr Exp $
 **/

public class StringVector {
  /**
   *  Create a new vector of String objects.
   **/
  public StringVector () {
    vector = new Vector ();
  }

  /**
   *  Create a new vector which contains only the given String.
   **/
  public StringVector (final String new_string) {
    vector = new Vector ();
    add (new_string);
  }

  /**
   *  Performs the same function as Vector.addElement ()
   **/
  public void add (final String node) {
    vector.addElement (node);
  }

  /**
   *  Call add () on each of the String objects in the given StringVector.
   **/
  public void add (final StringVector new_strings) {
    for (int i = 0 ; i < new_strings.size () ; ++i) {
      add (new_strings.elementAt (i));
    }
  }

  /**
   *  Performs the same function as Vector.removeElement ()
   **/
  public boolean remove (final String node) {
    return vector.removeElement (node);
  }
  
  /**
   *  Performs the same function as Vector.elementAt ()
   **/
  public String elementAt (final int index) {
    return (String) vector.elementAt (index);
  }

  /**
   *  Performs the same function as Vector.setElementAt ()
   **/
  public void setElementAt (final String string, final int index) {
    vector.setElementAt (string, index);
  }

  /**
   *  Performs the same function as Vector.size ()
   **/
  public int size () {
    return vector.size ();
  }

  /**
   *  Searches for the first occurence of the given argument, testing for
   *  equality using the equals method.
   *  @return the index of the first occurrence of the argument in this
   *    vector; returns -1 if the object is not found.
   **/
  public int indexOf (final String string) {
    return vector.indexOf (string);
  }

  /**
   *  Return true if this object contains the given String, testing for
   *  equality using the equals method.
   **/
  public boolean contains (final String string) {
    if (indexOf (string) == -1) {
      return false;
    } else {
      return true;
    }
  }

  /**
   * Sorts the elements of the vector using a simple O(n^2) selection
   * sort.
   */
  public void sort () {
    int smallest;

    for (int i = 0; i < size (); ++i) {
      //find smallest remaining element
      smallest = i;
      for (int j = i + 1 ; j < size () ; ++j) {
        if (elementAt(j).compareTo (elementAt(smallest)) < 0) {
          smallest = j;
        }
      }
      //exchange smallest and i
      if (smallest != i) {
        final String tmp = elementAt (i);
        setElementAt (elementAt(smallest), i);
        setElementAt (tmp, smallest);
      }
    }
  }

  /**
   *  Return a new copy of this object.
   **/
  public StringVector copy () { 
    final StringVector new_string_vector = new StringVector ();

    new_string_vector.vector = (Vector) vector.clone ();

    return new_string_vector;
  }

  /**
   *  Return a StringVector containing the values of the given String after
   *  splitting using the given characters.  If the argument String is zero
   *  length or it consists only of the characters used to split, the return
   *  vector will be zero length.
   **/
  public static StringVector getStrings (final String argument,
                                         final String split_characters) {
    final StringVector return_vector = new StringVector ();

    final StringTokenizer tokeniser =
      new StringTokenizer (argument, split_characters);

    while (tokeniser.hasMoreTokens ()) {
      final String value = tokeniser.nextToken ();

      if (value.length () > 0) {
        return_vector.add (value);
      }
    }
    
    return return_vector;
  }

  /**
   *  Return a StringVector containing the values of the given String after
   *  splitting on whitespace.  The return object contains one String for each
   *  sequence of non-whitespace characters in the argument.  If the argument
   *  String is zero length or it consists only of whitespace, the return
   *  vector will be zero length.
   **/
  public static StringVector getStrings (final String argument) {
    return getStrings (argument, " \t");
  }

  /**
   *  Storage for String objects.
   **/
  private Vector vector;
}
